-- Copyright 2012 Alberto Sáez
--
--
-- This file is part of Witchaven the Roguelike.
--
--    Witchaven the Roguelike is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    Witchaven the Roguelike is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with Witchaven the Roguelike.  If not, see <http://www.gnu.org/licenses/>.

enemy = {}
enemyList = {}
enemyBlockMap = {}

function enemy.add(name, class, hostile, char, eColor, active, 
					awakeMessage, killMessage, catchMessage, attackMessage,
					x, y, hp, damage, ranged, rangedAttack)
					
	table.insert(enemyList, {name = name, class = class,  hostile = hostile, char = char, 
						 eColor = eColor, active = active,
						 awakeMessage = awakeMessage, killMessage = killMessage, catchMessage = catchMessage, attackMessage = attackMessage,
						 x = x, y = y,
						 hp = hp, damage = damage, ranged = ranged, rangedAttack = rangedAttack})
						 
	enemy.recalculateBlockMap()
end

function enemy.addPrototype(prototypeID, x, y)
	local prototype = enemiesList[prototypeID]
	local personality = protoPersonality[prototype.person]
	
	enemy.add(prototype.name, prototype.class, prototype.hostile, prototype.char, prototype.eColor, false, 
			  personality.awakeMessage, personality.killMessage, personality.catchMessage, personality.attackMessage,
			  x, y,
			  math.floor(prototype.hp + player.depth/1.5), math.floor(prototype.damage + player.depth/3 - 1),
			  enemiesList[prototypeID].ranged, enemiesList[prototypeID].rangedAttack)
			  
end

function enemy.recalculateBlockMap()
	for i = 1, mapHeight do
		enemyBlockMap[i] = {}
		for j = 1, mapWidth do
			
			if map[i][j] == "#" then
				enemyBlockMap[i][j] = "#"
			else
				enemyBlockMap[i][j] = "·"
			end
		end
	end
end

function enemy.draw()
	love.graphics.setFont(myFont2)
	
	for i, v in ipairs(enemyList) do
		if showMobsOnMap == false then
			if viewMap[v.y][v.x] == 1 then
				color(v.eColor)
				lprint(v.char, v.x*(fontSize - 5), v.y*(fontSize - 5) + 3)
			end
		else
			color(v.eColor)
			lprint(v.char, v.x*(fontSize - 5), v.y*(fontSize - 5) + 3)
		end
	end
end

function enemy.exists(x, y)
	local exists = false
	for i,v in ipairs(enemyList) do
		if v.x == x and v.y == y then
			exists = true
			break
		end
	end
	
	return exists
end

function enemy.awake()
	local message = nil
	
	if player.invis <= 0 then
		for i, v in ipairs(enemyList) do
			if v.active == false and viewMap[v.y][v.x] == 1 then
				if string.find("aiueoAIUEO", string.sub(v.name, 1, 1)) ~= nil then
					message = "An " .. v.name .. " comes into view"
				else
					message = "A " .. v.name .. " comes into view"
				end
				
				if v.awakeMessage ~= "" then
					message = message .. ": \"" .. v.awakeMessage .. "\""
				end
				
				logMessage(message, v.eColor)
				
				v.active = true
			end
		end
	end
end

function enemy.distanceToPlayer()
	for i,v in ipairs(enemyList) do
		if game.distance(v.x, v.y, player.x, player.y) <= 3 and v.active == true then
			if math.random(1, 100) <= 5 and v.catchMessage ~= "" then
				logMessage(v.name .. " > \"" .. v.catchMessage .. "\"", v.eColor)
			end
		end
	end
end

function enemy.kill(enemyID)
	local newExperience = nil
	
	newExperience = enemyList[enemyID].damage*6 + math.floor(enemyList[enemyID].damage*math.random(0, 50)/100)
	
	player.exp = player.exp + newExperience
	table.remove(enemyList, enemyID)
	
	enemy.recalculateBlockMap()
	
	logMessage("You obtained " .. newExperience .. " experience points.", {150, 255, 0, 255})
	
	player.shootX = 0
	player.shootY = 0
end

function enemy.move()
	local jumper = require("jumper")
	pather = jumper(enemyBlockMap, "·")
	pather:setMode("ORTHOGONAL")
	
	for i, v in ipairs(enemyList) do
		doTurn = false
		if v.active == true then
			myPath = pather:getPath(v.x, v.y, player.x, player.y)
			
			if  myPath ~= nil then
				if myPath[2].x == player.x and myPath[2].y == player.y and v.ranged == false then
					enemy.attack(v)
				elseif viewMap[v.y][v.x] == 1 and v.ranged == true then
					if v.rangedAttack == "Arrow" then
						projectiles.add(v.x, v.y, player.x, player.y, v.damage, v.name .. "'s " .. v.rangedAttack, true)
					else
						projectiles.add(v.x, v.y, player.x, player.y, v.damage, v.name .. "'s " .. v.rangedAttack, false, {255,0,0,255})
					end
				else
					blockedPath = enemy.checkFreeCell(myPath[2].x, myPath[2].y, i)
					if blockedPath == false then
						v.x = myPath[2].x
						v.y = myPath[2].y
					elseif blockedPath == true then
						enemy.alternatePath(v)
					end
				end
			else
				-- can't move
			end
		end
	end
end

function enemy.attack(thisEnemy)
	logMessage("The " .. thisEnemy.name .. " attacked you.", {255,0,0,255})
	player.inflictDamage(thisEnemy.damage)
	if math.random(1,100) <= 10 and thisEnemy.attackMessage ~= "" then
		logMessage(thisEnemy.name .. " > \"" .. thisEnemy.attackMessage .. "\"", thisEnemy.eColor)
	end
	
	if thisEnemy.name == "Rat" then
		if math.random(1, 100) <= 10 then
			player.poison = player.poison + math.random(5, 7)
			logMessage("You feel something running through your veins...", {255, 0, 0, 255})
		end
	end
	
	if thisEnemy.name == "Imp" then
		if math.random(1, 100) <= 35 then
			player.poison = player.poison + math.random(4, 10)
			logMessage("You feel something running through your veins...", {255, 0, 0, 255})
		end
	end
end

function enemy.checkFreeCell(x, y, thisID)
	local pathBlock = false
	
	for i, v in ipairs(enemyList) do
		if i ~= thisID and v.x == x and v.y == y then
			pathBlock = true
			break
		end
	end
	
	return pathBlock
end

function enemy.alternatePath(enemy)
	local tempBlockMap = enemyBlockMap
	local tempJumper = require("jumper")
	
	for i,v in ipairs(enemyList) do
		--if v.x ~= enemy.x and v.y ~= enemy.y then
			tempBlockMap[v.y][v.x] = "#"
		--end
	end
	
	tempPather = tempJumper(tempBlockMap, "·")
	tempPather:setMode("ORTHOGONAL")
	
	tempPather = tempPather:getPath(enemy.x, enemy.y, player.x, player.y)
	
	if tempPather ~= nil then
		enemy.x = tempPather[2].x
		enemy.y = tempPather[2].y
	end
end

function enemy.checkStatus()
	for i,v in ipairs(enemyList) do
		if v.hp <= 0 then
			if v.name == "Cirae-Argoth" then
				game.win()
			else
				if v.killMessage ~= "" then
					logMessage(v.name .. " > \"" .. v.killMessage .. "\"", v.eColor)
				end
				
				logMessage("The " .. v.name .. " dies.", {255, 255, 0, 255})
				
				if map[v.y][v.x] == "·" then
					if math.random(1, 100) <= corpseChance then
						map[v.y][v.x] = "%"
					else
						local bloodyRemains = nil
						local remains = math.random(1,3)
						
						if remains == 1 then
							bloodyRemains = ","
						elseif remains == 2 then
							bloodyRemains = ":"
						elseif remains == 3 then
							bloodyRemains = "\""
						end
						
						map[v.y][v.x] = bloodyRemains
					end
				end
			end
			enemy.kill(i)
		end
	end
end

function enemy.save()
	local saveEnemy = "enemyList = "
	
	saveEnemy = saveEnemy .. ttostring(enemyList)
	saveEnemy = saveEnemy .. "\nenemyBlockMap = " .. ttostring(enemyBlockMap)
	
	success = love.filesystem.write("enemy.save", saveEnemy)
	
	return success
end

function enemy.load()
	ok, chunk = pcall(love.filesystem.load, "enemy.save")
	return ok, chunk
end
