-- Copyright 2012 Alberto Sáez
--
--
-- This file is part of Witchaven the Roguelike.
--
--    Witchaven the Roguelike is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    Witchaven the Roguelike is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with Witchaven the Roguelike.  If not, see <http://www.gnu.org/licenses/>.


game = {}

game.status = "idle"

game.lookX = 0
game.lookY = 0

game.overlyAlpha = 255

function game.kill(panic)
	-- ok = love.filesystem.remove("enemy.save")
	-- ok = love.filesystem.remove("game.map")
	-- ok = love.filesystem.remove("mapItems.save")
	-- ok = love.filesystem.remove("playerItems.save")
	-- ok = love.filesystem.remove("player.save")
	
	if panic == true then
		print("Saving and quitting...")
		love.timer.sleep(1)
		game.save()
		print("Done. Sayonara.")
	end
	
	love.mouse.setVisible(true)
	love.event.push("quit")
end

function game.drawDeathOverly()
	color(255, game.overlyAlpha - 50, game.overlyAlpha - 50, game.overlyAlpha)
	love.graphics.rectangle("fill", 0, 0, screenWidth, screenHeight)
	
	if game.overlyAlpha > 100 then
		game.overlyAlpha = game.overlyAlpha - 10
	end
end

function game.save()
	logMessage("Suspending game...", {100,100,100,255})
	
	aux = player.save()
	aux2 = m.save()
	aux3 = enemy.save()
	aux4 = inventory.saveMapItems()
	aux5 = inventory.savePlayerItems()
	
	if aux and aux2 and aux3 and aux4 and aux5 then
		logMessage("Game Suspended", {100,100,100,255})
	else
		
		logMessage("Can't suspend game", {100,100,100,255})
	end
end

function game.checkLoad()
	if love.filesystem.exists("enemy.save") and love.filesystem.exists("game.map") and love.filesystem.exists("mapItems.save") and 
		love.filesystem.exists("player.save") and love.filesystem.exists("playerItems.save") then
		
		game.load()
	end
end

function game.load()
	logMessage("Restarting suspended game...", {100,100,100,255})
	
	aux, aix = player.load()
	aux2, aix2 = m.load()
	aux3, aix3 = enemy.load()
	aux4, aix4 = inventory.loadMapItems()
	aux5, aix5 = inventory.loadPlayerItems()
	
	if aux and aux2 and aux3 and aux4 and aux5 then
		aix()
		aix2()
		aix3()
		aix4()
		aix5()
		logMessage("Game restarted.", {100,100,100,255})
		
	else
		logMessage("Can't load previously suspended game.", {100,100,100,255})
	end
	
	ok = love.filesystem.remove("enemy.save")
	ok = love.filesystem.remove("game.map")
	ok = love.filesystem.remove("mapItems.save")
	ok = love.filesystem.remove("playerItems.save")
	ok = love.filesystem.remove("player.save")
	
	game.setStatus("idle")
	
	restartColor()
end

function game.win()
	game.overlyAlpha = 255
	game.setStatus("win")
end

function game.drawTipsOverly()
	aux = love.graphics.newImage("helpScreen.png")
	
	color(255, 255, 255, 255)
	love.graphics.draw(aux, 0, 0)
end

function game.drawWinOverly()
	aux = love.graphics.newImage("end.png")
	
	rndColor = math.random(200, 255)
	
	color(rndColor, rndColor, rndColor, 255)
	love.graphics.draw(aux, 0, 0)
	
	
	color(255, 255, 255, game.overlyAlpha)
	
	love.graphics.rectangle("fill", 0, 0, screenWidth, screenHeight)
	
	if game.overlyAlpha > 0 then
		game.overlyAlpha = game.overlyAlpha - 20
	end
	
	if game.overlyAlpha < 0 then game.overlyAlpha = 0 end
end

function game.restart(onlyNewMap)
	map = {}
	rooms = {}
	doors = {}
	corridors = {}
	enemyList = {}
	enemyBlockMap = {}
	mapItems = {}
	
	if onlyNewMap ~= true then
		player.level = 1
		player.exp = 0
		player.depth = 1
		player.expToLevel = 0
		player.maxHealth = 0
		
		player.baseStr = 0
		
		player.recalculateStats()
		
		player.health = player.maxHealth
		
		player.strMod = 0
		player.poison = 0
		player.invis = 0
		player.vision = 0
		player.shrineMod = 0
		
		player.jop = 0
		player.jol = 0
		player.joe = 0
		
		playerWeapons = {}
		inventory.addWeapon(1, weapons[1].bdmg, math.random(0, 1), math.random(55, 75), true)
		inventory.addWeapon(2, weapons[2].bdmg, math.random(0, 1), math.random(55, 75), true)
		inventory.addWeapon(3, weapons[3].bdmg, math.random(0, 1), math.random(55, 75), true)
		inventory.addWeapon(4, weapons[4].bdmg, math.random(0, 1), math.random(55, 75), true)
		
		--inventory.addWeapon(5, weapons[5].bdmg, math.random(0, 1), math.random(55, 75), true)
		--inventory.addWeapon(6, weapons[6].bdmg, math.random(0, 1), math.random(55, 75), true)
		--inventory.addWeapon(7, weapons[7].bdmg, math.random(0, 1), math.random(55, 75), true)
		--inventory.addWeapon(8, weapons[8].bdmg, math.random(0, 1), math.random(55, 75), true)
		--inventory.addWeapon(9, weapons[8].bdmg, math.random(0, 1), math.random(55, 75), true)
		
		player.selectedWeapon = 1
		player.arrows = math.random(4, 5)
		player.axes = math.random(3, 6)
		
		-- durability: zero is broken
		-- defense: damage reduction
		-- resistance: chance to loss dur
		playerArmor = {durability = 100, defense = 20, resistance = 70}
		
		player.spells[1] = 0
		player.spells[2] = 0
		player.spells[3] = 0
		player.spells[4] = 4
		player.spells[5] = 0
		player.spells[6] = 2
		player.spells[7] = 0
		player.spells[8] = 1

		playerPotions[1] = 4
		playerPotions[2] = 1
		playerPotions[3] = 2
		playerPotions[4] = 0
		playerPotions[5] = 1
		
		player.shootX = 0
		player.shootY = 0
	end
	
	player.hasPentagram = false
	player.usedEnchanting = false
	
	m.generate(11, 3, 6)
	m.generateItems(player.depth*2, 4)
	m.generateEnemies()
	m.generateDecor()
	
	restartColor()
end

function game.showHelp()
	aux = love.filesystem.read("help.me")
	
	love.graphics.setFont(HUDFont)
	color(175,175,175,255)
	lprint(aux, 5, 5)
end

function game.getStatus()
	return game.status
end

function game.setStatus(status)
	game.status = status
end

function game.showDescription(x, y)
	local location = ""
	if viewMap[y][x] == 0 then
		location = location .. "Unexplored location"
	else
		location = location .. mapDescriptions[map[y][x]]
		
		for i,v in ipairs(enemyList) do
			if v.x == x and v.y == y then
				if string.find("aiueoAIUEO", string.sub(v.name, 1, 1)) ~= nil then
					location = location .. ", an " .. v.name .. " (" .. v.class 
				else
					location = location .. ", a " .. v.name .. " (" .. v.class 
				end
				if v.hostile == true then
					location = location .. ", hostile"
				end
				location = location .. ")"
			end
		end
		
		for i,v in ipairs(mapItems) do
			local thisItemDesc = nil
			
			if v.x == x and v.y == y then
				if string.find("aiueoAIUEO", string.sub(mapDescriptions[v.char], 1, 1)) ~= nil then
					location = location .. ", you see here an " .. mapDescriptions[v.char]
				else
					location = location .. ", you see here a " .. mapDescriptions[v.char]
				end
				
				if v.iType == "w" then
					location = location .. " (" .. weapons[v.attr1].name
					thisItemDesc = true
				end
				
				if v.iType == "p" then
					location = location .. " (" .. potions[v.attr1].name
					thisItemDesc = true
				end
				
				if v.iType == "a" then
					location = location .. " (" .. armorNames[tostring(v.attr1)]
					thisItemDesc = true
				end
				
				if v.iType == "s" then
					location = location .. " (" .. spellNames[v.attr1]
					thisItemDesc = true
				end
				
				if v.iType == "j" then
					location = location .. " (" .. shardNames[v.attr1]
					thisItemDesc = true
				end
				
				if thisItemDesc ~= nil then
					if v.identified == false and v.attr3 ~= 0 then
						location = location .. ", Unidentified)"
					else
						if v.attr3 < 0 then
							location = location .. " " .. v.attr3 .. ")"
						elseif v.attr3 > 0 then
							location = location .. " +" .. v.attr3 .. ")"
						else
							location = location .. ")"
						end
					end
				end
			end
		end
	end
	
	if player.x == x and player.y == y then
		location = location .. ", You are here"
	end
	
	logMessage(location)
end


function game.turn()
	player.turns = player.turns + 1
	enemy.checkStatus()
	enemy.distanceToPlayer()
	enemy.move()
	
	player.checkStatus()
end


function game.distance(x0, y0, x1, y1)
	return math.floor(    math.sqrt(    math.abs(x0 - x1)^2 + math.abs(y0 - y1)^2    )    )
end
