-- Copyright 2012 Alberto Sáez
--
--
-- This file is part of Witchaven the Roguelike.
--
--    Witchaven the Roguelike is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    Witchaven the Roguelike is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with Witchaven the Roguelike.  If not, see <http://www.gnu.org/licenses/>.

inventory = {}
mapItems = {}

function inventory.addWeapon(weaponNum, baseDamage, modifier, durability, identified)
	if identified == nil then identified = false end
	if durability == nil then durability = 100 end
	
	table.insert(playerWeapons, {weaponNum = weaponNum, baseDamage = baseDamage, modifier = modifier, durability = durability, identified = identified})
end

function inventory.addItemToMap(char, iColor, x, y, iType, identified, attr1, attr2, attr3, attr4)
	table.insert(mapItems, {char = char, iColor = iColor, x = x, y = y, iType = iType, identified = identified, attr1 = attr1, attr2 = attr2, attr3 = attr3, attr4 = attr4})
end

function inventory.drawMapItems()
	love.graphics.setFont(myFont2)
	
	for i, v in ipairs(mapItems) do
		if showItemsOnMap == false then
			if viewMap[v.y][v.x] == 1 then
				color(v.iColor)
				lprint(v.char, v.x*(fontSize - 5), v.y*(fontSize - 5) + 3)
			end
		else
			color(v.iColor)
			lprint(v.char, v.x*(fontSize - 5), v.y*(fontSize - 5) + 3)
		end
	end
end

function inventory.saveMapItems()
	local saveMapItems = nil
	
	saveMapItems = "mapItems = " .. ttostring(mapItems)
	
	success = love.filesystem.write("mapItems.save", saveMapItems)
	
	return success
end

function inventory.loadMapItems()
	ok, chunk = pcall(love.filesystem.load, "mapItems.save" )
	
	return ok, chunk
end

function inventory.savePlayerItems()
	local savePlayerItems = nil
	
	savePlayerItems = "playerWeapons = " .. ttostring(playerWeapons)
	
	success = love.filesystem.write("playerItems.save", savePlayerItems)
	
	return success
end

function inventory.loadPlayerItems()
	ok, chunk = pcall(love.filesystem.load, "playerItems.save" )
	
	return ok, chunk
end

function inventory.drawWeapon()
	local hudLine = 0
	
	love.graphics.setFont(HUDFont)
	
	for i,v in ipairs(playerWeapons) do
		local identified = v.identified
		if i == player.selectedWeapon then
			color (255,55 + v.durability*2,55 + v.durability*2,255)
		else
			color (100,v.durability,v.durability,255)
		end
		
		if identified == false then identified = "?" else 
			if v.modifier < 0 then
				identified = tostring(v.modifier)
			else
				identified = "+" .. tostring(v.modifier)
			end
		end
		
		if v.modifier == 0 then
			lprint( "[" .. i .. "] " .. weapons[v.weaponNum].name .. " BDmg = " .. v.baseDamage, screenWidth-280, 390 + hudLine)
		else
			lprint( "[" .. i .. "] " .. weapons[v.weaponNum].name .. " " .. identified .. " BDmg = " .. v.baseDamage, screenWidth-280, 390 + hudLine)
		end
		
		hudLine = hudLine + 15
	end
end
