-- Copyright 2012 Alberto Sáez
--
--
-- This file is part of Witchaven the Roguelike.
--
--    Witchaven the Roguelike is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    Witchaven the Roguelike is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with Witchaven the Roguelike.  If not, see <http://www.gnu.org/licenses/>.

require "data"
require "map"
require "player"
require "playerStats"
require "var"
require "jumper"
require "enemy"
require "los"
require "gui"
require "game"
require "inventory"
require "projectiles"

function gameArgs()
--#################################
--arg[2] = "-n" -- USED ONLY FOR DEVEL PURPOSES!!!!!
--#################################
	local argument = nil
	if arg[2] == "--morgue" or arg[2] == "-m" or arg[1] == "--morgue" or arg[1] == "-m" then
		
		ok, chunk = pcall(love.filesystem.read, "game.morgue" )
		
		if ok then
			argument = "\n\n\nWitchaven the Roguelike - Morgue File\n\n" .. chunk
			print(argument)
		else
			argument = "I can't find any morgue file!!!"
		end
	elseif arg[2] == "--new" or arg[2] == "-n" or arg[1] == "--new" or arg[1] == "-n" then
		argument = "newgame"
	elseif arg[2] == "--help" or arg[2] == "-h" or arg[1] == "--help" or arg[1] == "-h" then
		print("\nWIRL launch options can be:\n-h,  --help       Show this help screen\n-m,  --morgue     Dumpts morgue file to stdout\n-n,  --new        Starts a new game instead of loading possible suspended game")
		argument = "help"
	end
	
	return argument
end

function love.load()
	aux = gameArgs()
	
	if aux == nil or aux == "newgame" then
		love.graphics.setMode(screenWidth,screenHeight,false)
		
		love.graphics.setBackgroundColor(0, 0, 0, 255)
		
		game.restart(false)
		
		logMessage("Welcome to " .. __APPNAME)
		logMessage("Version is " .. __VER)
		logMessage("Press ^h for commands help")
		
		game.setStatus("tips")
		
		if aux ~= "newgame" then
			game.checkLoad()
		end
		
		love.mouse.setVisible(false)
	else
		game.kill()
	end
	
	love.keyboard.setKeyRepeat(0.2, 0.05)
end

function love.draw()
	if game.getStatus() ~= "help" then
		m.draw()
		enemy.draw()
		inventory.drawMapItems()
		player.draw()
		gui.draw()
		drawMessages()
		projectiles.move()
	else
		game.showHelp()
	end
	
	if game.getStatus() == "look" then
		color(255,0,0,255)
		lprint("X", game.lookX*(fontSize-5), game.lookY*(fontSize-5) + 3)
	end
	
	if game.getStatus() == "shoot" then
		color(255,0,0,255)
		lprint("X", player.shootX*(fontSize-5)+1, player.shootY*(fontSize-5)+5)
	end
	
	if game.getStatus() == "dead" then
		game.drawDeathOverly()
	end
	
	if game.getStatus() == "win" then
		game.drawWinOverly()
	end
	
	if game.getStatus() == "tips" then
		game.drawTipsOverly()
	end
end

function love.update(dt)
	-- this looks so empty....
end

function love.keypressed(key, unicode)
	if game.getStatus() == "idle" then
		if key == "a" then 
			if previousKey == "lctrl" then
				game.restart(false)
			else
				m.applyItem()
			end
		elseif key == "," then
			player.pickupItem()
		elseif key == "p" then
			if previousKey == "lctrl" then
				player.applyJOP()
			end
		elseif (key == "1" or key == "2" or key == "3" or key == "4" or key == "5") and (previousKey == "lalt") then
			local potionNum = unicode - 48
			player.usePotion(tonumber(potionNum))
		elseif key == "f1" or key == "f2" or key == "f3" or key == "f4" or key == "f5" or key == "f6" or key == "f7" or key == "f8" then
			local spellNum = string.sub(key, 2)
			player.castSpell(tonumber(spellNum))
			
			if player.shootX == 0 then
				player.shootX = player.x
				player.shootY = player.y
			end
		elseif key == "f9" then 
			showLogTime = not(showLogTime)
			logMessage("Show turns message timer = " .. tostring(showLogTime))
		elseif key == "f12" then
			if previousKey == "lctrl" then
				showMobsOnMap = not(showMobsOnMap)
				previousKey = ""
				logMessage ("Show enemies on map = " .. tostring(showMobsOnMap))
			end
		elseif key == "f11" then
			if previousKey == "lctrl" then
				showItemsOnMap = not(showItemsOnMap)
				previousKey = ""
				logMessage ("Show items on map = " .. tostring(showItemsOnMap))
			end
		elseif key == "f10" then
			if previousKey == "lctrl" then
				showLevelMap = not(showLevelMap)
				previousKey = ""
				logMessage ("Reveal level map = " .. tostring(showLevelMap))
			end
		elseif key == "u" then
			if previousKey == "lctrl" then
				game.setStatus("tips")
			end
		elseif key == "q" then
			if previousKey == "lctrl" then
				game.kill()
			end
		elseif key == "escape" then
			game.kill(true)
		elseif key == "." then
			game.turn()
		elseif key == "h" then
			if previousKey == "lctrl" then
				game.setStatus("help")
			else
				player.move(-1, 0)
			end
		elseif key == "e" then
			if previousKey == "lctrl" then
				player.applyJOE()
			end
		elseif unicode >= 49 and unicode <= 57 then
			player.changeWeapon(unicode - 48)
		elseif key == "c" then
			if previousKey == "lctrl" then
				colorMode = colorMode + 1
				if colorMode > 3 then colorMode = 1 end
					
				if colorMode == 2 then
					logMessage("Color mode = Black and White")
				elseif colorMode == 3 then
					logMessage("Color mode = Random")
					
				else
					logMessage("Color mode = Trip to Hell")
				end
				restartColor()
			end
		elseif key == "t" then
			if previousKey == "lctrl" then
				logMessage("Select target to shoot an arrow", {200, 200, 100, 255})
				game.setStatus("shoot")
				player.shootMode = "arrow"
				if player.shootX == 0 then
					player.shootX = player.x
					player.shootY = player.y
				end
			else
				logMessage("Select target to throw a Pike Axe", {200, 200, 100, 255})
				game.setStatus("shoot")
				player.shootMode = "axe"
				if player.shootX == 0 then
					player.shootX = player.x
					player.shootY = player.y
				end
			end
		elseif key == "s" then
			player.searchCells()
		elseif key == "up" or key == "k" then
			player.move(0, -1)
		elseif key == "down" or key == "j" then
			player.move(0, 1)
		elseif key == "left" then
			player.move(-1, 0)
		elseif key == "right" or key == "l" then
			player.move(1, 0)
		elseif key == "o" then
			game.lookX = player.x
			game.lookY = player.y
			game.showDescription(player.x, player.y)
			game.setStatus("look")
		elseif key == "i" then
			if previousKey == "lctrl" then
				player.applyJOL()
			end
		end
		
		
	elseif game.getStatus() == "shoot" then
		if key == "return" then
			if player.shootMode == "arrow" then
				player.shootArrow()
			elseif player.shootMode == "axe" then
				player.shootAxe()
			elseif player.shootMode == 4 or player.shootMode == 6 then
				player.shootSpell()
			end
		elseif key == "escape" then
			game.setStatus("idle")
			player.shootX = 0
			player.shootY = 0
		elseif key == ("up") then
			player.shootY = player.shootY - 1
		elseif key == ("down") then
			player.shootY = player.shootY + 1
		elseif key == ("left") then
			player.shootX = player.shootX - 1
		elseif key == ("right") then
			player.shootX = player.shootX + 1
		end
		
		
	elseif game.getStatus() == "look" then
		if key == ("up") then
			game.lookY = game.lookY - 1
			game.showDescription(game.lookX, game.lookY)
		elseif key == ("down") then
			game.lookY = game.lookY + 1
			game.showDescription(game.lookX, game.lookY)
		elseif key == ("left") then
			game.lookX = game.lookX - 1
			game.showDescription(game.lookX, game.lookY)
		elseif key == ("right") then
			game.lookX = game.lookX + 1
			game.showDescription(game.lookX, game.lookY)
		elseif key == "escape" then
			game.setStatus("idle")
		end
		
		
	elseif game.getStatus() == "help" then
		if key == "escape" then
			game.setStatus("idle")
		end
		
	elseif game.getStatus() == "tips" then
		if key == "escape" then
			game.setStatus("idle")
		end
		
	elseif game.getStatus() == "dead" then
		if key == "escape" then
			game.kill()
		end
		
	elseif game.getStatus() == "win" then
		if key == "escape" then
			game.kill()
		end
	end
	
	if key == "f" then
		if previousKey == "lctrl" then
			fullScreen = not(fullScreen)
			love.graphics.setMode( screenWidth, screenHeight, fullScreen)
		end
	elseif key == "v" then
		logMessage("Version is " .. __VER)
	end
	
	previousKey = key
end

function ttostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
      table.insert( result, key_to_str( k ) .. "=" .. val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end

function val_to_str ( v )
  if "string" == type( v ) then
    v = string.gsub( v, "\n", "\\n" )
    if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
      return "'" .. v .. "'"
    end
    return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
  else
    return "table" == type( v ) and ttostring( v ) or
      tostring( v )
  end
end

function key_to_str ( k )
  if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
    return k
  else
    return "[" .. val_to_str( k ) .. "]"
  end
end

function restartColor()
	if colorMode == 1 then
		floorColorLOS = {60 + player.depth*3, 20 + player.depth*2, 0 + player.depth, 255}
		wallColorLOS = {255, 170 - player.depth*10, 100 - player.depth*5, 255}
		floorColorFOW = {30 + player.depth, 15 - player.depth, 5, 255}
		wallColorFOW = {70 + player.depth, 45 - player.depth, 20 - player.depth, 255}
		decorColorLOS = {255, 50, 0 , 255}
		decorColorFOW = {100, 5, 0, 255}
	elseif colorMode == 2 then
		floorColorLOS = {100,100,100, 255}
		wallColorLOS = {200,200,200, 255}
		floorColorFOW = {25,25,25, 255}
		wallColorFOW = {60,60,60, 255}
		decorColorLOS = {200,200,200 , 255}
		decorColorFOW = {50,50,50, 255}
	elseif colorMode == 3 then
		local r1, g1, b1, r2, g2, b2 = math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(0, 255), math.random(0, 255)
		floorColorLOS = {r1/2,g1/2,b1/2, 255}
		wallColorLOS = {r2, g2, b2, 255}
		floorColorFOW = {r1/10, g1/10, b1/10, 255}
		wallColorFOW = {r2/6, g2/6, b2/6, 255}
		decorColorLOS = {200,200,200, 255}
		decorColorFOW = {50,50,50, 255}
	end
end

function logMessage(message, mColor)
	if message ~= nil then
		if mColor == nil then
			mColor = {175,175,175,255}
		end
		if showLogTime then message = "[T." .. player.turns .. "]> " .. message end
		
		table.insert(messagesLog, {message = message, mColor = mColor})
	else
		table.insert(messagesLog, {message = "THERE SHOULD BE A MESSAGE HERE!!!", mColor = {0,255,255,255}})
	end
end

function drawMessages()
	local lineNum = 1
	love.graphics.setFont(HUDFont)
	
	for i, v in ipairs(messagesLog) do
		if i > table.getn(messagesLog) - messageLines then
			color(v.mColor)
			lprint(v.message, 15, screenHeight - fontSize*lineNum - 10)
			lineNum = lineNum + 1
		end
	end
end

function showMessagesLog()
	
end
