-- Copyright 2012 Alberto Sáez
--
--
-- This file is part of Witchaven the Roguelike.
--
--    Witchaven the Roguelike is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    Witchaven the Roguelike is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with Witchaven the Roguelike.  If not, see <http://www.gnu.org/licenses/>.

player = {}
player.spells = {}

playerWeapons = {}
playerPotions = {}
playerArmor = {}

player.x = 0
player.y = 0
player.turns = 0

player.shootX = 0
player.shootY = 0

player.shootMode = nil

function player.draw()
	color(50,100,255,255)
	love.graphics.setFont(myFont2)
	lprint("@", player.x*(fontSize - 5), player.y*(fontSize - 5))
end

function player.move(x, y)
	if map[player.y + y][player.x + x] ~= "#" then
		if enemy.exists(player.x + x, player.y + y) == false then
			player.x = player.x + x
			player.y = player.y + y
			
			if map[player.y][player.x] == ";" then
				logMessage("The magma floor hurts you a lot!!!", {255,0,0, 255})
				player.health = player.health - 10
			end
		else
			player.attack(player.x + x, player.y + y)
		end
		
		if traps[player.y][player.x] == "^" then
			map[player.y][player.x] = "^"
			traps[player.y][player.x] = "·"
			
			local trapType = math.random(1, 5)
			
			if trapType == 1 then
				logMessage("Suddently, a needle is thrown at you. Lukily, it was not poisonous.", {255, 0, 0, 255})
				player.health = player.health - math.random(10, 15)
			elseif trapType == 2 then
				logMessage("A lance comes out from a small opening in the wall. You can't avoid the damage!!!", {255, 0, 0, 255})
				player.health = player.health - math.floor(player.health*math.random(20, 25)/100)
			elseif trapType == 3 then
				logMessage("Suddently, a trapdoor opens and you fall to a pit damaging your armor heavily", {255, 0, 0, 255})
				playerArmor.durability = playerArmor.durability - math.random(15, 20)
				player.depth = player.depth + 1
				game.restart(true)
			elseif trapType == 4 then
				logMessage("As you walk a step, you hear a 'click' sound. Soon, you are surrounded by poisonous gases", {255, 0, 0, 255})
				player.poison = math.random(10, 20)
			elseif trapType == 5 then
				logMessage("A boulder falls upon you but you block part of the damage with your weapon...", {255, 0, 0, 255})
				playerWeapons[player.selectedWeapon].durability = playerWeapons[player.selectedWeapon].durability - math.random(15, 20)
				
				if playerWeapons[player.selectedWeapon].durability < 0 then playerWeapons[player.selectedWeapon].durability = 0 end
			end
		end
		
		game.turn()
		
		if math.random(1, 300) <= 1 then
			logMessage(ambience[math.random(1, table.getn(ambience))], {255,255,255,255})
		end
	end
end

function player.changeWeapon(newWeapon)
	if playerWeapons[newWeapon] ~= nil then
		if newWeapon ~= player.selectedWeapon then
			if playerWeapons[newWeapon].durability > 0 and playerWeapons[newWeapon].durability <= 10 then
				logMessage("WARNING: it is almost broken!!!", {255, 0, 255, 255})
			elseif playerWeapons[newWeapon].durability <= 0 then
				logMessage("WARNING: It is broken and you will fight bare handed!!!", {255, 0, 255, 255})
			end
		end
		
		player.selectedWeapon = newWeapon
		logMessage("You ready the " .. weapons[playerWeapons[newWeapon].weaponNum].name , {255, 0, 255, 255})
		game.turn()
	else
		logMessage("You don't have such weapon")
	end
end

function player.attack(x, y)
	local playerDamage = nil
	local damageModifier = playerWeapons[player.selectedWeapon].modifier
	local breakChance = weapons[player.selectedWeapon].breakChance
	local selectedWeapon = playerWeapons[player.selectedWeapon].weaponNum
	
	player.invis = 0
	
	if player.strMod > 0 then
		playerDamage = playerWeapons[player.selectedWeapon].baseDamage + player.baseStr
		playerDamage = playerDamage + playerDamage*50/100
	else
		playerDamage = playerWeapons[player.selectedWeapon].baseDamage + player.baseStr
	end
	
	if playerWeapons[player.selectedWeapon].durability <= 0 then
		playerDamage = 1
		damageModifier = 0
		logMessage("Your actual weapon is broken!!! You are fighting bare handed!!!", {255,255,0,255})
	end
	
	if playerWeapons[player.selectedWeapon].weaponNum == 6 then
		local weaponDamaged = math.random(10, 20)
		playerWeapons[player.selectedWeapon].durability = playerWeapons[player.selectedWeapon].durability - weaponDamaged
				
		logMessage("You are attacking close combat with your bow!!! It is receiving heavy damage!!!", {255,255,0,255})
	end
	
	if player.shrineMod > 0 then
		playerDamage = playerDamage + playerDamage/2
	end
	
	playerDamage = math.floor(playerDamage)
	
	for i, v in ipairs(enemyList) do
		if v.x == x and v.y == y then
			if playerDamage + damageModifier > 0 then
				logMessage("You attack the " .. v.name .. " with your " .. weapons[selectedWeapon].name)
				v.hp = v.hp - playerDamage - damageModifier
			else
				logMessage("You attack the " .. v.name ..  " with your " .. weapons[selectedWeapon].name .. ", but he didn't received damage!!!")
			end
			
			if math.random(0,100) <= breakChance and player.selectedWeapon ~= 0 and playerWeapons[player.selectedWeapon].durability > 0 and 
				playerWeapons[player.selectedWeapon].weaponNum ~= 6 then
				
				local weaponDamaged = math.random(1, 5)
				playerWeapons[player.selectedWeapon].durability = playerWeapons[player.selectedWeapon].durability - weaponDamaged
				
				logMessage("Your " .. weapons[selectedWeapon].name .. " has been damaged " .. weaponDamages[weaponDamaged], {0, 100,255,255})
				
				if playerWeapons[player.selectedWeapon].durability < 0 then playerWeapons[player.selectedWeapon].durability = 0 end
			end
		end
	end
end

function player.inflictDamage(damage)
	if playerArmor.durability > 0 then
		damage = math.floor(damage - damage*playerArmor.defense/100)
		
		if math.random(1, 100) <= playerArmor.resistance then
			playerArmor.durability = playerArmor.durability - 1
		end
	end
	
	player.health = player.health - damage
end

function player.searchCells()
	local x, y = player.x, player.y
	
	if traps[y-1][x-1] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y-1][x-1] = "·"
		map[y-1][x-1] = "^"
	elseif traps[y-1][x] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y-1][x] = "·"
		map[y-1][x] = "^"
	elseif traps[y-1][x+1] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y-1][x+1] = "·"
		map[y-1][x+1] = "^"
	elseif traps[y][x-1] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y][x-1] = "·"
		map[y][x-1] = "^"
	elseif traps[y][x+1] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y][x+1] = "·"
		map[y][x+1] = "^"
	elseif traps[y+1][x-1] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y+1][x-1] = "·"
		map[y+1][x-1] = "^"
	elseif traps[y+1][x] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y+1][x] = "·"
		map[y+1][x] ="^"
	elseif traps[y+1][x+1] == "^" then
		logMessage("You detected a trap near you!!!", {255,0,0,255})
		traps[y+1][x+1] = "·"
		map[y+1][x+1] = "^"
	else
		logMessage("You don't see any trap near you")
	end
	
	game.turn()
end

function player.pickupItem()
	local gotSomething = false
	
	for i,v in ipairs(mapItems) do
		if v.x == player.x and v.y == player.y then
			gotSomething = true
			
			aux = player.selectivePickUp(v, i)
			
			logMessage(aux, {50, 100, 255, 255})
			
			game.turn()
		end
	end
	
	if map[player.y][player.x] == "%" then
		player.lootCorpse()
		gotSomething = true
		game.turn()
	end
	
	if gotSomething == false then
		logMessage("There is nothing to pick up here.", {200, 100, 50, 255})
	end
end

function player.lootCorpse()
	local message = "You search the corpse ... ... ... "
	local randomItem = math.random(1, 100)
	
	if randomItem <= 10 then
		-- weapon
		local thisWeapon = math.random(1, 9)
		local thisModifier = math.random(1, 100)
		
		message = message .. " You found a weapon!!!"
		
		if thisModifier <= 25 then
			inventory.addItemToMap(")", {150, 150, 150, 255}, player.x, player.y, "w", false, thisWeapon, weapons[thisWeapon].bdmg, math.random(1, 3), math.random(60, 100))
		elseif thisModifier > 25 and thisModifier < 80 then
			inventory.addItemToMap(")", {150, 150, 150, 255}, player.x, player.y, "w", false, thisWeapon, weapons[thisWeapon].bdmg, 0, math.random(60, 100))
		else
			inventory.addItemToMap(")", {150, 150, 150, 255}, player.x, player.y, "w", false, thisWeapon, weapons[thisWeapon].bdmg, math.random(-3, -1), math.random(60, 100))
		end
	elseif randomItem > 10 and randomItem <= 20 then
		-- scroll
		inventory.addItemToMap("*", {200, 120, 25, 255}, player.x, player.y, "s", true, math.random(1, 8), 0, 0, 0)
		message = message .. " You found a magic scroll!!!"
	elseif randomItem > 20 and randomItem <= 30 then
		-- armor
		inventory.addItemToMap("[", {150, 150, 150, 255}, player.x, player.y, "a", true, math.random(2, 4)*10, math.random(80, 100), 0, 0)
		message = message .. " You found an armor!!!"
	elseif randomItem > 30 and randomItem <= 45 then
		-- potion
		aux = math.random(1, 5)
		inventory.addItemToMap("!", potions[aux].color, player.x, player.y, "p", true, aux, 0, 0, 0)
		message = message .. " You found a potion!!!"
	elseif randomItem > 45 and randomItem <= 60 then
		-- gold
		inventory.addItemToMap("$", {255, 255, 0, 255}, player.x, player.y, "g", true, math.random(player.level*5, 5*math.floor(player.level + player.level/2)), 0, 0, 0)
		message = message .. " You found some gold coins!!!"
	elseif randomItem > 60 and randomItem <= 80 then
		-- nothing
		message = message .. " But you see nothing useful"
	elseif randomItem > 80 and randomItem <= 90 then
		-- arrow
		inventory.addItemToMap("/", {235, 180, 80, 255}, player.x, player.y, "r", true, 0, 0, 0, 0)
		message = message .. " You found an arrow."
	elseif randomItem > 90 then
		-- enchant jewel
		message = message .. " But you see nothing useful"
	end
	
	local remains = math.random(1,3)
					
	if remains == 1 then
		bloodyRemains = ","
	elseif remains == 2 then
		bloodyRemains = ":"
	elseif remains == 3 then
		bloodyRemains = "\""
	end
	
	map[player.y][player.x] = bloodyRemains
	
	logMessage(message, {50,100, 200, 255})
end

function player.selectivePickUp(thisItem, itemID)
	local attr1, attr2, attr3, attr4, identified = nil, nil, nil, nil, nil
	local returnString = nil
	local weaponAvailable = false
	
	if thisItem.iType == "w" then
		for i,v in ipairs(playerWeapons) do
			
			if v.weaponNum == thisItem.attr1 then
				weaponAvailable = true
				
				attr1 = v.weaponNum
				attr2 = v.baseDamage
				attr3 = v.modifier
				attr4 = v.durability
				identified = v.identified
				
				v.baseDamage = thisItem.attr2
				v.modifier = thisItem.attr3
				v.durability = thisItem.attr4
				v.identified = thisItem.identified
				
				thisItem.char = ")"
				thisItem.iColor = {150, 150, 150, 255}
				thisItem.x = player.x
				thisItem.y = player.y
				thisItem.identified = identified
				thisItem.attr1 = attr1
				thisItem.attr2 = attr2
				thisItem.attr3 = attr3
				thisItem.attr4 = attr4
				
				if thisItem.attr1 == 7 then
					player.axes = player.axes + math.random(3, 5)
					if player.axes > 10 then player.axes = 10 end
				end
				
				if v.identified == false and v.modifier ~= 0 then
					returnString = "You pick up the Unidentified " .. weapons[v.weaponNum].name
				else
					if v.modifier ~= 0 then
						if v.modifier > 0 then 
							returnString = "You pick up the " .. weapons[v.weaponNum].name .. " +" .. v.modifier
						else
							returnString = "You pick up the " .. weapons[v.weaponNum].name .. " " .. v.modifier
						end
					else
						returnString = "You pick up the " .. weapons[v.weaponNum].name
					end
				end
			end
		end
		
		if weaponAvailable == false then
			inventory.addWeapon(thisItem.attr1, thisItem.attr2, thisItem.attr3, thisItem.attr4, thisItem.identified)
			
			if thisItem.identified == false then
				returnString = "You pick up the " .. "Unidentified " .. weapons[thisItem.attr1].name
			else
				if thisItem.attr3 ~= 0 then
					if v.modifier > 0 then 
							returnString = "You pick up the " .. weapons[v.weaponNum].name .. " +" .. v.modifier
						else
							returnString = "You pick up the " .. weapons[v.weaponNum].name .. " " .. v.modifier
						end
				else
					returnString = weapons[thisItem.attr1].name
				end
			end
			
			table.remove(mapItems, itemID)
		end
	elseif thisItem.iType == "p" then
		playerPotions[thisItem.attr1] = playerPotions[thisItem.attr1] + 1
		returnString = "You got the " .. potions[thisItem.attr1].name .. " potion."
		
		table.remove(mapItems, itemID)
	elseif thisItem.iType == "g" then
		player.exp = player.exp + thisItem.attr1
		
		returnString = "You pick up the " .. thisItem.attr1 .. " gold coins"
		
		table.remove(mapItems, itemID)
		
	elseif thisItem.iType == "s" then
		if player.spells[thisItem.attr1] >= 10 then
			returnString = "You can't carry more " .. spellNames[thisItem.attr1] .. " scrolls"
		else
			player.spells[thisItem.attr1] = player.spells[thisItem.attr1] + 1
			returnString = "You got the " .. spellNames[thisItem.attr1] .. " scroll"
			table.remove(mapItems, itemID)
		end
	elseif thisItem.iType == "a" then
		local thisArmor = armorNames[tostring(thisItem.attr1)]
		local mapArmor = {}
		
		if playerArmor.durability > 0 then
			mapArmor.attr1 = thisItem.attr1
			mapArmor.attr2 = thisItem.attr2
			
			thisItem.attr1 = playerArmor.defense
			thisItem.attr2 = playerArmor.durability
			thisItem.iColor = {150,150,150,255}
			
			playerArmor.defense = mapArmor.attr1
			playerArmor.resistance = armorNames[thisArmor]
			playerArmor.durability = mapArmor.attr2
		else
			playerArmor.defense = thisItem.attr1
			playerArmor.resistance = armorNames[thisArmor]
			playerArmor.durability = thisItem.attr2
			
			table.remove(mapItems, itemID)
		end
		
		returnString = "You wear the " .. thisArmor
		
	elseif thisItem.iType == "t" then
		table.remove(mapItems, itemID)
		
		player.hasPentagram = true
		
		returnString = "You pick up the Teleporting Pentagram!!!"
		
	elseif thisItem.iType == "r" then
		if player.arrows >= 10 then
			returnString = "You can't carry more arrows."
		else
			player.arrows = player.arrows + 1
			table.remove(mapItems, itemID)
			returnString = "You obtained an Arrow"
		end
		
	elseif thisItem.iType == "j" then
		if thisItem.attr1 == 1 then
			player.jop = player.jop + 1
		elseif thisItem.attr1 == 2 then
			player.jol = player.jol + 1
		else
			player.joe = player.joe + 1
		end
		
		returnString = "You pick found a " .. shardNames[thisItem.attr1] .. " !!!"
		
		table.remove(mapItems, itemID)
	end
	
	return returnString
end

function player.shootArrow()
	if player.arrows > 0 then
		if playerWeapons[player.selectedWeapon].weaponNum == 6 then
			if player.x ~= player.shootX or player.y ~= player.shootY then
				local playerBow = playerWeapons[player.selectedWeapon]
				local bowDamage = nil
				
				if playerWeapons[player.selectedWeapon].durability > 0 then
					bowDamage = playerBow.baseDamage
				else
					bowDamage = playerBow.baseDamage/2
					logMessage("Your bow is broken. Ranged damage is halved.", {255,255,0,255})
				end
				
				projectiles.add(player.x, player.y, player.shootX, player.shootY, bowDamage + playerBow.modifier + player.strMod, "Your arrow hits", true, {235, 180, 80, 255})
				
				player.arrows = player.arrows - 1
			else
				logMessage("You can't shoot to yourself!!!")
			end
		else
			logMessage("You need to select first the bow before shooting arrows")
		end
	else
		logMessage("You don't have any arrow left!!!", {255, 255, 0, 255})
	end
	
	game.setStatus("idle")
	
	doTurn = true
end

function player.shootAxe()
	if player.axes > 0 then
		if playerWeapons[player.selectedWeapon].weaponNum == 7 then
			if player.x ~= player.shootX or player.y ~= player.shootY then
				local playerAxe = playerWeapons[player.selectedWeapon]
				local axeDamage = nil
				
				axeDamage = playerAxe.baseDamage
				
				projectiles.add(player.x, player.y, player.shootX, player.shootY, axeDamage + playerAxe.modifier + player.strMod + 5, "Your Pike axe hits", false, {200, 200, 200, 255})
				
				player.axes = player.axes - 1
			else
				logMessage("You can't shoot to yourself!!!")
			end
		else
			logMessage("You need to select first the Pike Axe before throwing them")
		end
	else
		logMessage("You can't throw your last pickaxe!!!", {200, 200, 200, 255})
	end
	
	game.setStatus("idle")
	
	doTurn = true
end

function player.shootSpell()
	local spellDamage = nil
	local spellColor = nil
	
	if player.shootMode == 4 then
		spellDamage = math.floor(player.shootMode + player.level/2 + player.depth*2.2)
	else
		spellDamage = math.floor(player.shootMode + player.level*2 + player.depth/1.5)
	end
	
	if player.shootMode == 4 then 
		spellColor = {125,205,195, 255}
	else
		spellColor = {255, 110, 0, 255}
	end
	
	projectiles.add(player.x, player.y, player.shootX, player.shootY, spellDamage , "Your " .. spellNames[player.shootMode] .. " hits", false, spellColor)
	player.spells[player.shootMode] = player.spells[player.shootMode] - 1
	
	game.setStatus("idle")
	
	doTurn = true
end

function player.applyJOP()
	if player.jop > 0 then
		playerWeapons[player.selectedWeapon].modifier = playerWeapons[player.selectedWeapon].modifier + 1
		logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " glows red for a moment", {255,0,0,255})
		logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " is now filled with magic", {255,0,0,255})
		player.jop = player.jop - 1
		game.turn()
	else
		logMessage("You don't have any Shard of Power left", {255,0,0,255})
	end
end

function player.applyJOL()
	if player.jol > 0 then
		playerWeapons[player.selectedWeapon].baseDamage = playerWeapons[player.selectedWeapon].baseDamage + math.random(-1, 2)
		playerWeapons[player.selectedWeapon].modifier = playerWeapons[player.selectedWeapon].modifier + math.random(-1, 2)
		logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " glows white for a moment", {0, 255, 175, 255})
		logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " is now bloodthirsty", {0, 255, 175, 255})
		player.jol = player.jol - 1
		game.turn()
	else
		logMessage("You don't have any Shard of Life left", {0, 255, 175, 255})
	end
end

function player.applyJOE()
	if player.joe > 0 then
		playerWeapons[player.selectedWeapon].durability = playerWeapons[player.selectedWeapon].durability + 25
			
		if playerWeapons[player.selectedWeapon].durability >= 100 then
			playerWeapons[player.selectedWeapon].durability = 100
			logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " glows magenta for a moment, but it seems that something is wrong...", {255, 125, 255, 255})
		else
			logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " glows magenta for a moment", {255, 125, 255, 255})
			logMessage("Your " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " seems to go back in time", {255, 125, 255, 255})
		end
		player.joe = player.joe - 1
		game.turn()
	else
		logMessage("You don't have any Shard of Eternity left", {255, 125, 255, 255})
	end
end

function player.save()
	local savePlayer = nil
	
	savePlayer = "player.x = " .. player.x
	savePlayer = savePlayer .. "\nplayer.y = " .. player.y
	
	savePlayer = savePlayer .. "\nplayer.level = " .. player.level
	savePlayer = savePlayer .. "\nplayer.expToLevel = " .. player.expToLevel
	savePlayer = savePlayer .. "\nplayer.exp = " .. player.exp
	
	savePlayer = savePlayer .. "\nplayer.turns = " .. player.turns
	savePlayer = savePlayer .. "\nplayer.selectedWeapon = " .. player.selectedWeapon
	
	savePlayer = savePlayer .. "\nplayer.maxHealth = " .. player.maxHealth
	savePlayer = savePlayer .. "\nplayer.health = " .. player.health
	
	savePlayer = savePlayer .. "\nplayer.baseStr = " .. player.baseStr
	savePlayer = savePlayer .. "\nplayer.strMod = " .. player.strMod
	
	savePlayer = savePlayer .. "\nplayer.spells[1] = " .. player.spells[1]
	savePlayer = savePlayer .. "\nplayer.spells[2] = " .. player.spells[2]
	savePlayer = savePlayer .. "\nplayer.spells[3] = " .. player.spells[3]
	savePlayer = savePlayer .. "\nplayer.spells[4] = " .. player.spells[4]
	savePlayer = savePlayer .. "\nplayer.spells[5] = " .. player.spells[5]
	savePlayer = savePlayer .. "\nplayer.spells[6] = " .. player.spells[6]
	savePlayer = savePlayer .. "\nplayer.spells[7] = " .. player.spells[7]
	
	savePlayer = savePlayer .. "\nplayerPotions[1] = " .. playerPotions[1]
	savePlayer = savePlayer .. "\nplayerPotions[2] = " .. playerPotions[2]
	savePlayer = savePlayer .. "\nplayerPotions[3] = " .. playerPotions[3]
	savePlayer = savePlayer .. "\nplayerPotions[4] = " .. playerPotions[4]
	savePlayer = savePlayer .. "\nplayerPotions[5] = " .. playerPotions[5]
	
	savePlayer = savePlayer .. "\nplayerArmor = " .. ttostring(playerArmor)
	
	savePlayer = savePlayer .. "\nplayer.hasPentagram = " .. tostring(player.hasPentagram)
	savePlayer = savePlayer .. "\nplayer.usedEnchanting = " .. tostring(player.usedEnchanting)
	savePlayer = savePlayer .. "\nplayer.shrineMod = " .. tostring(player.shrineMod)
	
	savePlayer = savePlayer .. "\nplayer.depth = " .. player.depth

	savePlayer = savePlayer .. "\nplayer.arrows = " .. player.arrows
	savePlayer = savePlayer .. "\nplayer.axes = " .. player.axes
	
	savePlayer = savePlayer .. "\nplayer.poison = " .. player.poison
	savePlayer = savePlayer .. "\nplayer.invis = " .. player.invis
	savePlayer = savePlayer .. "\nplayer.vision = " .. player.vision
	
	savePlayer = savePlayer .. "\nplayer.jop = " .. player.jop
	savePlayer = savePlayer .. "\nplayer.jol = " .. player.jol
	savePlayer = savePlayer .. "\nplayer.joe = " .. player.joe
	
	success = love.filesystem.write("player.save", savePlayer)
	
	return success
end

function player.load()
	ok, chunk = pcall(love.filesystem.load, "player.save" )
	
	return ok, chunk
end

function player.checkStatus()
	if player.poison > 0 then
		player.health = player.health - 1
		player.poison = player.poison - 1
		logMessage("Poison runs through your veins!!!", {255, 160, 100,255})
	end
	
	if player.invis > 0 then
		player.invis = player.invis - 1
		
		if player.invis <= 0 then
			logMessage("You return to normal status", {190,85,0,255})
			player.invis = 0
		end
	end
	
	if player.vision > 0 then 
		player.vision = player.vision - 1
		if player.vision <= 0 then
			logMessage("Your eyes can't see through mists anymore", {190,190,255,255})
			player.vision = 0
			playerLOS = 10
		end
	end
	
	if player.strMod > 0 then 
		player.strMod = player.strMod - 1
		
		if player.strMod <= 0 then
			logMessage("Your strength is no longer titanic", {100, 255, 100,255})
			player.strMod = 0
		end
	end
	
	if player.shrineMod > 0 then 
		player.shrineMod = player.shrineMod - 1
		
		if player.shrineMod <= 0 then
			logMessage("Your weapons stop glowing", {0, 255, 200,255})
			player.shrineMod = 0
		end
	end
	
	if player.health < 0 then player.health = 0 end
	
	if player.health <= 0 and playerPotions[1] > 0 then 
		player.usePotion(1)
	elseif player.health <= 0 and playerPotions[1] <= 0 then
		local thisGame = nil
		
		logMessage("YOU DIE. Press [ESC] to exit", {255, 255, 0, 255})
		
		player.visitMorgue()
		
		game.setStatus("dead")
	end
	
	if player.exp >= player.expToLevel then
		player.levelUp()
	end
	
	-- Health recovery??? NO without food or water
	--if player.health > 0 then
		--if player.turns % 10 == 0 then
			--player.health = player.health + 1
			--if player.health > player.maxHealth then player.health = player.maxHealth end
		--end
	--end
end

function player.levelUp()
	if player.exp == player.expToLevel then
		player.exp = 0
		player.level = player.level + 1
	elseif player.exp > player.expToLevel then
		player.exp = player.exp - player.expToLevel
		player.level = player.level + 1
	end
	
	player.recalculateStats()
	
	logMessage("You are now level " .. player.level .. "!!!", {255,195,100})
end

function player.visitMorgue()
	ok, chunk = pcall(love.filesystem.read, "game.morgue" )
	if ok ~= true then
		chunk = ""
	end
	
	thisGame = "\n\n····-----=======#######=======-----····"
	thisGame = thisGame .. "\n\n" .. tostring(os.date("%c"))
	thisGame = thisGame .. "\n\n" .. morgueMessage[math.random(1,6)]
	thisGame = thisGame .. "\n\nPlayer has fallen somewhere in depth " .. player.depth ..  " after " .. player.turns .. " turns."
	thisGame = thisGame .. "\nHe was level " .. player.level .. " and only needed " .. player.expToLevel - player.exp .. " experience points to level up."
	thisGame = thisGame .. "\n\nHe was wielding a " .. weapons[playerWeapons[player.selectedWeapon].weaponNum].name .. " but still, he couldn't stand the fight."
	thisGame = thisGame .. "\n\nHis weapons were:"
	for i, v in ipairs(playerWeapons) do
		thisGame = thisGame .. "\n  [" .. i .. "]  " .. weapons[v.weaponNum].name
		if v.modifier < 0 then
			thisGame = thisGame .. " " .. v.modifier
		elseif v.modifier > 0 then
			thisGame = thisGame .. " +" .. v.modifier
		end
		
		if v.identified ~= true and v.modifier ~= 0 then
			thisGame = thisGame .. " (Unidentified)"
		end
		
		thisGame = thisGame .. " Remain durability = " .. v.durability .. "%"
	end
	
	if playerArmor.durability > 0 then
		thisGame = thisGame .. "\nHe were wearing a " .. armorNames[tostring(playerArmor.defense)]
	end
	
	thisGame = thisGame .. "\n\nHe were carrying the next potions:"
	thisGame = thisGame .. "\n  " .. playerPotions[1] .. " potions of Healing"
	thisGame = thisGame .. "\n  " .. playerPotions[2] .. " potions of Strength"
	thisGame = thisGame .. "\n  " .. playerPotions[3] .. " bottles of Antidote"
	thisGame = thisGame .. "\n  " .. playerPotions[4] .. " potions of Fire Resistance"
	thisGame = thisGame .. "\n  " .. playerPotions[5] .. " potions of Invisibility"
	
	thisGame = thisGame .. "\n\nHe had the next spell scrolls:"
	thisGame = thisGame .. "\n  Scare: " .. player.spells[1]
	thisGame = thisGame .. "\n  Night Vision: " .. player.spells[2]
	thisGame = thisGame .. "\n  Freeze: " .. player.spells[3]
	thisGame = thisGame .. "\n  Magic Arrow: " .. player.spells[4]
	thisGame = thisGame .. "\n  Fly: " .. player.spells[5]
	thisGame = thisGame .. "\n  Fireball: " .. player.spells[6]
	thisGame = thisGame .. "\n  Nuke: " .. player.spells[7]
	thisGame = thisGame .. "\n  Identify: " .. player.spells[8]
	
	thisGame = thisGame .. "\n\nThe last messages before he died were:"
	
	local topMessage = table.getn(messagesLog) - 25
	
	if topMessage <= 0 then
		topMessage = 1
	end
	
	for i = table.getn(messagesLog), topMessage, -1 do
		thisGame = thisGame .. "\n > " .. messagesLog[i].message
	end
	
	chunk = thisGame .. chunk
	
	love.filesystem.write("game.morgue", chunk)
end

function player.castSpell(spellNum)
	if player.spells[spellNum] <= 0 then
		logMessage("You don't have any scroll to cast " .. spellNames[spellNum] .. " spell")
	else
		if spellNum == 8 then
			-- Identify
			if playerWeapons[player.selectedWeapon].identified == true or playerWeapons[player.selectedWeapon].modifier == 0 then
				logMessage("That weapon is already identified.", {0, 255, 255, 255})
			else
				playerWeapons[player.selectedWeapon].identified = true
				logMessage("You understand now what your weapon is exactly", {0, 255, 255, 255})
				player.spells[spellNum] = player.spells[spellNum] - 1
				game.turn()
			end
		elseif spellNum == 6 or spellNum == 4 then
			game.setStatus("shoot")
			player.shootMode = spellNum
			logMessage("Select target to cast " .. spellNames[spellNum], {200, 200, 100, 255})
		elseif spellNum == 2 then
			player.vision = 200
			playerLOS = 20
			logMessage("Your eyes can now see through the mists...", {190,190, 255, 255})
			game.turn()
		elseif spellNum == 7 then
			logMessage("Your Nuke Spell calls a great destruction upon your enemies!!!", {255, 255, 255, 255})
			for i, v in ipairs(enemyList) do
				if viewMap[v.y][v.x] == 1 and
					v.name ~= "Cyrae-Argoth" then
					
					v.hp = v.hp - 50
				end
			end
			enemy.checkStatus()
			game.turn()
		end
	end
end

function player.recalculateStats()
	player.expToLevel = math.floor(    150 + (player.level ^ (player.level/2.75) )     )
		
	player.maxHealth = 90 + 5*player.level
	player.health = player.maxHealth
	
	player.baseStr = player.level/2.5
end

function player.usePotion(potionNum)
	local potionMessage = nil
	
	if playerPotions[potionNum] <= 0 then
		potionMessage = "You don't have any " .. potions[potionNum].name .. " potion."
	else
		if potionNum == 1 then
			if player.health < player.maxHealth then
				player.health = player.health + 25
				
				playerPotions[potionNum] = playerPotions[potionNum] - 1
				
				potionMessage = "You quaff a " .. potions[potionNum].name .. " potion."
				
				if player.health > player.maxHealth then player.health = player.maxHealth end
			else
				potionMessage = "You are already at full health. You no need a healing potion."
			end
		elseif potionNum == 2 then
			player.strMod = player.strMod + 50
			potionMessage = "You are now blessed with titans' strength."
			playerPotions[potionNum] = playerPotions[potionNum] - 1
		elseif potionNum == 3 then
			if player.poison > 0 then
				player.poison = 0
				potionMessage = "You blood is no longer poisoned."
				playerPotions[potionNum] = playerPotions[potionNum] - 1
			else
				potionMessage = "You no need an antidote."
			end
		elseif potionNum == 4 then
			potionMessage = "FIRE POTION"
		elseif potionNum == 5 then
			player.invis = player.invis + 100
			potionMessage = "You melt with light and shadows..."
			playerPotions[potionNum] = playerPotions[potionNum] - 1
		end
		game.turn()
	end
	
	logMessage(potionMessage, potions[potionNum].color)
end
